@@ -1,14 +1,7 @@
-from typing import (
-    Any,
-    Iterable,
-    Iterator,
-    List,
-    Mapping,
-    Sequence,
-    Set,
-    Tuple,
-    Union,
-)
+from __future__ import annotations
+
+from collections import namedtuple
+from typing import TYPE_CHECKING, Any, Iterator, Sequence, Tuple
 
 import pytest
 from packaging.requirements import Requirement
@@ -19,15 +12,21 @@ from resolvelib import (
     BaseReporter,
     InconsistentCandidate,
     ResolutionImpossible,
-    Resolver,
 )
 from resolvelib.resolvers import (
-    Criterion,
-    RequirementInformation,
     RequirementsConflicted,
     Resolution,
+    Resolver,
 )
 
+if TYPE_CHECKING:
+    from typing import Iterable, Mapping
+
+    from resolvelib.resolvers import (
+        Criterion,
+        RequirementInformation,
+    )
+
 
 def test_candidate_inconsistent_error():
     requirement = "foo"
@@ -115,10 +114,19 @@ def test_candidate_depends_on_requirements_of_same_identifier(specifiers):
 
 
 def test_resolving_conflicts():
+    Candidate = namedtuple(
+        "Candidate", ["name", "version", "requirements"]
+    )  # name, version, requirements
+    _Requirement = namedtuple("Requirement", ["name", "versions"])  # name, versions
+    a1 = Candidate("a", 1, [_Requirement("q", {1})])
+    a2 = Candidate("a", 2, [_Requirement("q", {2})])
+    b = Candidate("b", 1, [_Requirement("q", {1})])
+    q1 = Candidate("q", 1, [])
+    q2 = Candidate("q", 2, [])
     all_candidates = {
-        "a": [("a", 1, [("q", {1})]), ("a", 2, [("q", {2})])],
-        "b": [("b", 1, [("q", {1})])],
-        "q": [("q", 1, []), ("q", 2, [])],
+        "a": [a1, a2],
+        "b": [b],
+        "q": [q1, q2],
     }
 
     class Reporter(BaseReporter):
@@ -136,20 +144,20 @@ def test_resolving_conflicts():
             return 0
 
         def get_dependencies(self, candidate):
-            return candidate[2]
+            return candidate.requirements
 
         def find_matches(self, identifier, requirements, incompatibilities):
-            bad_versions = {c[1] for c in incompatibilities[identifier]}
+            bad_versions = {c.version for c in incompatibilities[identifier]}
             candidates = [
                 c
                 for c in all_candidates[identifier]
-                if all(c[1] in r[1] for r in requirements[identifier])
-                and c[1] not in bad_versions
+                if all(c.version in r.versions for r in requirements[identifier])
+                and c.version not in bad_versions
             ]
-            return sorted(candidates, key=lambda c: c[1], reverse=True)
+            return sorted(candidates, key=lambda c: c.version, reverse=True)
 
         def is_satisfied_by(self, requirement, candidate):
-            return candidate[1] in requirement[1]
+            return candidate.version in requirement.versions
 
     def run_resolver(*args):
         reporter = Reporter()
@@ -160,8 +168,10 @@ def test_resolving_conflicts():
         except ResolutionImpossible as e:
             return e.causes
 
-    backtracking_causes = run_resolver([("a", {1, 2}), ("b", {1})])
-    exception_causes = run_resolver([("a", {2}), ("b", {1})])
+    backtracking_causes = run_resolver(
+        [_Requirement("a", {1, 2}), _Requirement("b", {1})]
+    )
+    exception_causes = run_resolver([_Requirement("a", {2}), _Requirement("b", {1})])
     assert exception_causes == backtracking_causes
 
 
@@ -171,10 +181,9 @@ def test_pin_conflict_with_self(monkeypatch, reporter):
     Verify correct behavior of attempting to pin a candidate version that conflicts
     with a previously pinned (now invalidated) version for that same candidate (#91).
     """
-    Candidate = Tuple[
-        str, Version, Sequence[str]
-    ]  # name, version, requirements
-    all_candidates = {
+    Candidate = Tuple[str, Version, Sequence[str]]
+
+    all_candidates: Mapping[str, Sequence[Candidate]] = {
         "parent": [("parent", Version("1"), ["child<2"])],
         "child": [
             ("child", Version("2"), ["grandchild>=2"]),
@@ -185,11 +194,10 @@ def test_pin_conflict_with_self(monkeypatch, reporter):
             ("grandchild", Version("2"), []),
             ("grandchild", Version("1"), []),
         ],
-    }  # type: Mapping[str, Sequence[Candidate]]
+    }
 
-    class Provider(AbstractProvider):  # AbstractProvider[str, Candidate, str]
-        def identify(self, requirement_or_candidate):
-            # type: (Union[str, Candidate]) -> str
+    class Provider(AbstractProvider[str, Candidate, str]):
+        def identify(self, requirement_or_candidate: str | Candidate) -> str:
             result = (
                 Requirement(requirement_or_candidate).name
                 if isinstance(requirement_or_candidate, str)
@@ -198,22 +206,19 @@ def test_pin_conflict_with_self(monkeypatch, reporter):
             assert result in all_candidates, "unknown requirement_or_candidate"
             return result
 
-        def get_preference(self, identifier, *args, **kwargs):
-            # type: (str, *object, **object) -> str
+        def get_preference(self, identifier: str, *args: Any, **kwargs: Any) -> str:
             # prefer child over parent (alphabetically)
             return identifier
 
-        def get_dependencies(self, candidate):
-            # type: (Candidate) -> Sequence[str]
+        def get_dependencies(self, candidate: Candidate) -> Sequence[str]:
             return candidate[2]
 
         def find_matches(
             self,
-            identifier,  # type: str
-            requirements,  # type: Mapping[str, Iterator[str]]
-            incompatibilities,  # type: Mapping[str, Iterator[Candidate]]
-        ):
-            # type: (...) -> Iterator[Candidate]
+            identifier: str,
+            requirements: Mapping[str, Iterator[str]],
+            incompatibilities: Mapping[str, Iterator[Candidate]],
+        ) -> Iterator[Candidate]:
             return (
                 candidate
                 for candidate in all_candidates[identifier]
@@ -224,15 +229,12 @@ def test_pin_conflict_with_self(monkeypatch, reporter):
                 if candidate not in incompatibilities[identifier]
             )
 
-        def is_satisfied_by(self, requirement, candidate):
-            # type: (str, Candidate) -> bool
+        def is_satisfied_by(self, requirement: str, candidate: Candidate) -> bool:
             return candidate[1] in Requirement(requirement).specifier
 
     # patch Resolution._get_updated_criteria to collect rejected states
-    rejected_criteria = []  # type: List[Criterion]
-    get_updated_criteria_orig = (
-        Resolution._get_updated_criteria  # type: ignore[attr-defined]
-    )
+    rejected_criteria: list[Criterion] = []
+    get_updated_criteria_orig = Resolution._get_updated_criteria
 
     def get_updated_criteria_patch(self, candidate):
         try:
@@ -241,17 +243,14 @@ def test_pin_conflict_with_self(monkeypatch, reporter):
             rejected_criteria.append(e.criterion)
             raise
 
-    monkeypatch.setattr(
-        Resolution, "_get_updated_criteria", get_updated_criteria_patch
-    )
+    monkeypatch.setattr(Resolution, "_get_updated_criteria", get_updated_criteria_patch)
 
-    resolver = Resolver(
-        Provider(), reporter
-    )  # type: Resolver[str, Candidate, str]
+    resolver: Resolver[str, Candidate, str] = Resolver(Provider(), reporter)
     result = resolver.resolve(["child", "parent"])
 
-    def get_child_versions(information):
-        # type: (Iterable[RequirementInformation[str, Candidate]]) -> Set[str]
+    def get_child_versions(
+        information: Iterable[RequirementInformation[str, Candidate]],
+    ) -> set[str]:
         return {
             str(inf.parent[1])
             for inf in information
